@@ -16,16 +16,15 @@ class xodrStaticWorld():
         Its just handling the basic functionality needed for the current state of 
         automatum-data datasets.
 
+        :param path2XODR: Path to a xodr file. 
+
+        :var roads: A List of roads contained in the xodr. Roads are represtend by ``xodrRoad``.
+
     """
 
     def __init__(self, path2XODR):
         """
             Init function. 
-
-            Automatically reads the xodr and start to sample the lines into a 
-            cartesian coordinate system.
-
-            path2XODR: Path to a file in xodr format. 
         """
         self.roads = list()
 
@@ -37,7 +36,21 @@ class xodrStaticWorld():
             if "road" in child.tag:
                 self.roads.append(xodrRoad(child))
 
+    def calculate_point_to_lane_assignment(self, x, y):
+        """Calculates the lane assignment of a given point.
 
+        Therefore all roads are evaluated. 
+
+        :param x: Position X of Point
+        :param y: Position y of point
+        :return: Lane ID of the Lane, in which the point is. (Notice: 0 means no lane could be assigned, e.g. if the point is outside of the road.)
+        :return: Road ID in which the lane assignment is valid. (Notice: 0 means no assignment)
+        """
+        for road in self.roads:
+            lane_id = road.calculate_point_to_lane_assignment(x, y)
+            if(lane_id != 0):
+                return lane_id, road.id
+        return 0, 0
 
     def get_lane_marking_dicts(self, with_centerline = 0):
         """
@@ -55,9 +68,10 @@ class xodrStaticWorld():
         ]
         
         Types:
-        solid: Solid lane marking
-        dashed: Dashed lane marking
-        center_line: Center line fo the corresponding road element
+        
+        * solid: Solid lane marking
+        * dashed: Dashed lane marking
+        * center_line: Center line fo the corresponding road element
 
         with_centerline: controles if the center line is included in the list of lines, or if only real road markings are returned
 
@@ -72,8 +86,16 @@ class xodrRoad():
     """
         This class represents a road.
 
-        roadXML: XML-Tree of a road element
-        sampleWidth: Sample width that is used to calculate the lines a list of 
+        :param roadXML: XML-Tree of a road element
+        :param sampleWidth: Sample width that is used to calculate a plotable representation of the xodr.
+
+        :var name: Name of th road 
+        :var length: Length of the road
+        :var id: ID of Road (Currently unused, since automatum-data currently constit of one road)
+        :var junction: Information about Junctions (Currently unused, since automatum-data currently constit of one road)
+        :var geometry: Representation of reference line 
+        :var lanesections: Object that represents all lanes 
+
     """
 
     def __init__(self, roadXML, sampleWidth = 1.0):
@@ -110,6 +132,8 @@ class xodrRoad():
                         raise Exception("Unexpected XODR Format: Expected geometry, however given is %s"%laneChild.tag)
         self.sample_me(sampleWidth)
 
+
+
     def get_lane_marking_dicts(self):
         """
         This functions returns all lane markings corresponding to this static world object.
@@ -126,9 +150,10 @@ class xodrRoad():
         ]
 
         Types:
-        solid: Solid lane marking
-        dashed: Dashed lane marking
-        center_line: Center line fo the corresponding road element
+
+        * solid: Solid lane marking
+        * dashed: Dashed lane marking
+        * center_line: Center line fo the corresponding road element
             
         """
 
@@ -152,17 +177,71 @@ class xodrRoad():
     def __str__(self):
         """ Overwrite to string method"""
         return "Road %s length %.2f, ID: %d, Junction: %d"%(self.name, self.length, self.id, self.junction)
+
+    def calculate_point_to_lane_assignment(self, x, y):
+        """Calculates the lane assignment of a given point.
+
+        Therefore all roads are evaluated. 
+
+        :param x: Position X of Point
+        :param y: Position y of point
+        :return: Lane ID of the Lane, in which the point is. (Notice: 0 means no lane could be assigned, e.g. if the point is outside of the road.)
+        """
+        d, l, idx = self.get_track_and_orthogonal_distance(x, y)
+        if(idx is not None):
+            lane_section = self.lanesections[idx]
+            lane_id = lane_section.get_lane_id(d, l)
+            if(lane_id != 0):
+                return lane_id
+        return 0
+
+
+
+    def get_track_and_orthogonal_distance(self, x, y):
+        """Calculates for a given point the orthogonal distance from the road center line and the distance along the road:
+
+
+        :param x: Position X of Point
+        :param y: Position y of point
+        :return: Orthogonal distane to center line
+        :return: Distance along the road
+        :return: Idx of the geometry object where the point is assigned to
+        """
+        self.geometry.sort()
+        for seg_idx, geo in enumerate(self.geometry):
+            d, l = geo.get_track_and_orthogonal_distance(x, y)
+            if(l < geo.length):
+                return (d,l + geo.s, seg_idx)
+        return 0,0,None
+
+
         
 
 class xodrGeometry():
     """
-        Main class for all further geometry objects 
+        Main class for all further geometry objects. 
+
+        Based on xodr specification the following forms are supported by the corresponding child classes: 
+
+        * Straight Line by ``xodrGeometryLine``
+        * Arc by ``xodrGeometryCurvature``
+        * Spiral (not implemented)
+        * Cubic polynom (not implemented)
+        * Parametric cubic curve (not implemented)
+
+        :param xmlGeometry: xmlTree of a geometry object
+
+        :var s: Start distance of the Geometry 
+        :var x: Coordinate system x-position
+        :var y:  Coordinate system y-position
+        :var hdg:  Coordinate system heading
+        :var length: Relevant length of the geometry object
     """
     def __init__(self, xmlGeometry):
         """
             Init function that decodes the given xmlTree
             
-            xmlGeometry: xmlTree of a geometry object
+            
         """
         self.s = float(xmlGeometry.attrib['s'])
         self.x = float(xmlGeometry.attrib['x'])
@@ -189,16 +268,73 @@ class xodrGeometry():
         """ Sample of geometry object have to be overwritten by the child class"""
         raise NotImplementedError("Have to be overwritten by child class")
 
-
+    def get_track_and_orthogonal_distance(self, x, y):
+        """Calculates the distance between Reference Line alias Geometry and a given point"""
+        raise NotImplementedError("Have to be overwritten by child")
+    def __lt__(self, other):
+        """Overwrite left operator to enable a sorting based on start distance s
+        """
+        return self.s < other.s
 
 class xodrGeometryCurvature(xodrGeometry):
-    """ Geometry Class for a XODR Curvature"""
+    """ Geometry Class for a XODR Curvature
+
+    Overwrites the base class ``xodrGeometry``
+
+    :param xmlGeometry: xmlTree of a geometry object
+
+    :var curvature: Curvature of the reference line
+    """
     def __init__(self, xmlGeometry):
         """ Reads the Curvature relevant parameter and call the main class"""
 
 
         xodrGeometry.__init__(self, xmlGeometry) 
         self.curvature = float(xmlGeometry[0].attrib['curvature'])
+
+
+    def get_track_and_orthogonal_distance(self, x, y):
+        """Calculates the distance between Reference Line alias Geometry and a given point
+        
+
+        :param x: Position x of given Point
+        :param y: Position y of given point
+        
+        """
+        c, s = np.cos(-self.hdg), np.sin(-self.hdg)
+
+        R = np.array(((c, -s), (s, c)))         
+       
+        p = np.array([x - self.x,y - self.y])
+        sign_curvature = np.sign(self.curvature)
+        p_normed = np.matmul(R, p)
+        p_normed[1] = sign_curvature * p_normed[1]
+
+        r = 1/abs(self.curvature)
+
+        alpha = math.atan2(p_normed[0], r -p_normed[1])
+
+        if(alpha < 0 or alpha > math.pi/2):
+            return 0,0
+
+        l = r * alpha
+
+        c, s = np.cos(alpha), np.sin(alpha)
+
+
+        p_a = np.matmul(np.array(((c, -s), (s, c))), np.array((0, -r))) + np.array((0, r))
+        dx = p_a[0] - p_normed[0]
+        dy = p_a[1] - p_normed[1]
+        d = math.sqrt(dx**2 + dy**2)
+
+        dx = 0 - p_normed[0]
+        dy = r - p_normed[1]
+        d_middle_point = math.sqrt(dx**2 + dy**2)
+        if(d_middle_point < r):
+            d = -1 * d
+
+        return d, l
+
     def __str__(self):
         return "Curvature: curvature: %.2f, s: %.2f, x: %.2f, y: %.2f, hgd: %.2f, length %.2f"%(self.curvature, self.s, self.x, self.y, self.hdg, self.length)
 
@@ -236,15 +372,19 @@ class xodrGeometryCurvature(xodrGeometry):
             self.y_vec.append(global_point[1])
 
     def get_lane_marking_dicts(self):
-
+        """ Returns the lane marking plotting vector for the center line"""
         if(self.s_vec is None):
             raise Exception("Please call sample_me before activating the plotting")
 
         return [{"x_vec": list(self.x_vec), "y_vec": list(self.y_vec), "type": "centerline",  "color": "white", "width": 0.12}]
 
 
+
 class xodrGeometryLine(xodrGeometry):
-    """ Geometry Class for a XODR Line"""
+    """ Geometry Class for a XODR Line
+    
+    Notice: No further variables than xodrGeometry
+    """
     def __init__(self, xmlGeometry):
         """ Reads no additional and call the main class"""
         xodrGeometry.__init__(self, xmlGeometry) 
@@ -290,9 +430,10 @@ class xodrGeometryLine(xodrGeometry):
         ]
 
         Types:
-        solid: Solid lane marking
-        dashed: Dashed lane marking
-        center_line: Center line fo the corresponding road element
+        
+        * solid: Solid lane marking
+        * dashed: Dashed lane marking
+        * center_line: Center line fo the corresponding road element
             
         """
         if(self.s_vec is None):
@@ -300,11 +441,43 @@ class xodrGeometryLine(xodrGeometry):
 
         return [{"x_vec": list(self.x_vec), "y_vec": list(self.y_vec), "type": "centerline",  "color": "white", "width": 0.12}]
 
+    def get_track_and_orthogonal_distance(self, x, y):
+        """Calculates the distance between Reference Line alias Geometry and a given point
+        
+
+        :param x: Position x of given Point
+        :param y: Position y of given point
+        
+        """
+        c, s = np.cos(self.hdg), np.sin(self.hdg)
+
+        R = np.array(((c, -s), (s, c)))         
+        dir_vec = np.matmul(R, np.array([1,0]))
+       
+        p1=np.array([0,0])
+        p2=np.array([0,0]) + dir_vec
+        p3=np.array([x - self.x,y - self.y])
+        d=np.cross(p2-p1,p3-p1)/np.linalg.norm(p2-p1)
+
+
+        n = p2 - p1
+        v = p3 - p1
+
+        z = p1 + n*(np.dot(v, n)/np.dot(n, n))
+        l = math.sqrt(z[0] * z[0] + z[1] * z[1])
+
+
+        return d, l
 
  
 class xodrLaneSection():
     """
         Defines a XODR laneSection thats holds a number of lanes. 
+
+        :param xmlLaneSection: xmlTree of a lane section
+
+        :var s: Length of this lane section element along the reference line
+        :var lane: List of lane elements which belong to this lane section
     """
     def __init__(self, xmlLaneSection):
         """ Decodes the xmlTree of a LaneSection"""
@@ -319,6 +492,48 @@ class xodrLaneSection():
                     else:
                         raise Exception("Unexpected XODR Format: Expectation lane, given %s"%child.tag)
 
+    def get_lane_id(self, d, l):
+        """Returns the lane_id for a given point, defined by the distance l along the reference line 
+        and the orthogonal distance from the reference line. 
+
+        :param d: Orthogonal distance from reference line (signed according definition in xodr)
+        :param l: Distance along the reference line
+
+        :return: Id of the lane (0 if out of range)
+        """
+
+        posLines = list()
+        negLines = list()
+
+        for lane in self.lanes:
+            if(lane.id < 0):
+                negLines.append(lane)
+            else:
+                posLines.append(lane)
+
+        negLines.sort()
+        negLines.reverse()
+
+        posLines.sort()
+        
+        if(d > 0):
+            last_offset = 0
+            for lane in posLines:
+                lat_offset = lane.get_lateral_offset(l, offset_inner_lane=last_offset)
+                last_offset = lat_offset
+                if(lat_offset > d):
+                    return lane.id
+
+        if(d < 0):
+            last_offset = 0
+            for lane in negLines:
+                lat_offset = lane.get_lateral_offset(l, offset_inner_lane=last_offset)
+                last_offset = lat_offset
+                if(lat_offset < d):
+                    return lane.id
+        return 0
+
+
     def __str__(self):
         """
             Overwrites str()
@@ -342,9 +557,10 @@ class xodrLaneSection():
         ]
 
         Types:
-        solid: Solid lane marking
-        dashed: Dashed lane marking
-        center_line: Center line fo the corresponding road element
+        
+        * solid: Solid lane marking
+        * dashed: Dashed lane marking
+        * center_line: Center line fo the corresponding road element
             
         """        
         lineList = list()
@@ -383,8 +599,23 @@ class xodrLaneSection():
 class xodrLane():
     """
         Represents a XODR lane
+
+
+        :param xmlLane: xmlTree of a lane
+
+        :var id: ID of the Lane
+        :var type: Type of the lane, e.g. drivable or shoulder
+        :var width: Dict with all primate's of a third order polynom to define the width of the lane
+        :var roadmark: Dict of further information of road marks (Not completely set in automatum data)
+        :var material: Dict of further information of road material (Not set in automatum data)
+        :var speed: Dict of further information of speed limits (Not set in automatum data)
+
     """
     def __init__(self, xmlLane):
+        """Init function 
+
+        :param xmlLane: xmlTree of a lane
+        """
         self.id = int(xmlLane.attrib['id'])
         self.type = xmlLane.attrib['type']
         self.width = dict()
@@ -430,6 +661,17 @@ class xodrLane():
         """ Overwrite Compare operator to enable sorting of lines elements"""
         return self.id < other.id
 
+    def get_lateral_offset(self, s, offset_inner_lane=0):
+        """Returns the lateral offset for a given distance s along the lane
+
+        :param s: Distance along the lane
+        :param offset_inner_lane: Since the distance is given from the last lane and not from the reference line. This variables allows to give an constant offset on the lateral distance, defaults to 0
+
+        :return: Lateral offset of lane 
+
+        """
+        return (np.sign(self.id) * (self.width['a'] + self.width['b'] * s + self.width['c'] * s * s + self.width['d'] * s * s  * s)) + offset_inner_lane
+
     def sample_me(self, geo, innerLane = None):
         """ Samples this geometry object """  
         s_vec = copy.copy(geo.s_vec)
@@ -488,9 +730,10 @@ class xodrLane():
         ]
 
         Types:
-        solid: Solid lane marking
-        dashed: Dashed lane marking
-        center_line: Center line fo the corresponding road element
+        
+        * solid: Solid lane marking
+        * dashed: Dashed lane marking
+        * center_line: Center line fo the corresponding road element
             
         """              
         if(self.s_vec is None):
